/* Copyright 2004 Jetro Lauha - http://iki.fi/jetro/
 *
 * $Id: GameApp.cpp,v 1.3 2004/10/17 20:59:44 tonic Exp $
 * $Revision: 1.3 $
 */

#include "main.h"


GameApp::GameApp() :
    mStateInitStatus(TRUE),
    mGraphics(NULL)
{
}


GameApp::~GameApp()
{
    deinit();
    delete mGraphics;
}


BOOL GameApp::init()
{
    BOOL result = App::init();

    if (!result)
        return result;

    //mGraphics = new UIGraphicsOpenGLSDLCFL(gCFL);
    mGraphics = new UIGraphicsOpenGLSDL();
    if (mGraphics == NULL)
        return FALSE;

    UIFontOpenGLSDL *font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_treb_small.png");
    if (!font || !font->init())
        return FALSE;
    mFonts[FONT_SMALL] = font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_treb_medium.png");
    if (!font || !font->init())
        return FALSE;
    mFonts[FONT_MEDIUM] = font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_impact_big.png");
    if (!font || !font->init())
        return FALSE;
    mFonts[FONT_BIG] = font;

    font = new UIFontOpenGLSDL(mGraphics, DATA_PREFIX "font_nums.png", " $%'()*+,-./0123456789:;<=>");
    if (!font || !font->init())
        return FALSE;
    font->setCharacterSpacing(-4);
    font->setDrawMode(UIImage::NORMAL);
    mFonts[FONT_NUM] = font;

    initAndAddState(STATE_TITLE, new TitleState);
    initAndAddState(STATE_GAME_RUN, new GameState(mGraphics));
    if (!mStateInitStatus)
        return FALSE;

    setState(STATE_TITLE);
    //setState(STATE_GAME_RUN);

    return TRUE;
}


void GameApp::deinit()
{
    deinitAndDeleteStates();
    deleteFonts();
}


void GameApp::initAndAddState(STATE_TYPE stateId, State *newState)
{
    if (!mStateInitStatus)
        return;

    if (!newState->init())
    {
        mStateInitStatus = FALSE;
        delete newState;
        return;
    }
    else
        mStates[stateId] = newState;
}


void GameApp::deinitAndDeleteStates()
{
    StateMap::iterator it = mStates.begin();
    StateMap::iterator itend = mStates.end();
    for (; it != itend; ++it)
    {
        State *state = (*it).second;
        state->deinit();
        delete state;
    }
    mStates.clear();
}


void GameApp::deleteFonts()
{
    INT a;
    for (a = 0; a < FONTS; ++a)
        delete mFonts[a];
}


char * GameApp::getCaption()
{
    return CAPTION_STR;
}


State * GameApp::getStateHandler(STATE_TYPE state)
{
    State *res = mStates[state];
    assert(res != NULL);
    return res;
}
